#!/usr/bin/env python
# -*- coding: utf-8 -*-

# pycante.py
     
# Copyright 2011 Juan BC <jbc dot develop at gmail dot com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.


#===============================================================================
# DOC
#===============================================================================

"""The hotest way to deal with PyQt

Allows a unique way to deal with Qt Widgets QtDesigner or files. If the instance 
has a setupUI method is executed at the time of creation.

example:

    from PyQt4 import QtGui
    import pycante

    class MyWidget(pycante.E("/path/to/file.ui"), AnotherClass);
        pass
    
    class MyAnotherWidget(pycante.E(QtGui.QFrame), , AnotherClass):
        pass
        
    w0 = MyWidget()
    w1 = MyAnotherWidget()
    
    pycante.run()

"""

#===============================================================================
# META
#===============================================================================

__version__ = "0.1"
__license__ = "GPL3"
__author__ = "JBC"
__email__ = "jbc dot develop at gmail dot com"
__url__ = "https://bitbucket.org/leliel12/pycante"
__date__ = "2011-08-24"


#===============================================================================
# IMPORTS
#===============================================================================

import os, sys, inspect

from PyQt4 import QtGui
from PyQt4 import uic


#===============================================================================
# CONSTANS
#===============================================================================

APP = QtGui.QApplication(sys.argv)


#===============================================================================
# CLASS
#===============================================================================

class AutoSetup(object):
    """This class is used for auto execute 'setupUi' method if exist in a
    widget
    
    """
    
    def __init__(self, *args, **kwargs):
        super(AutoSetup, self).__init__(*args, **kwargs)
        if hasattr(self, "setupUi") and callable(self.setupUi):
            self.setupUi(self)


#===============================================================================
# DECORATOR
#===============================================================================

def E(ui_or_widget):
    """Resolve a qt widget class from ui file path or Widget class
    
    @param ui_or_widget for inerith visual stile (can be a designer ui file)
    @return New base class 
    
    Example:
    
        class MyWidget(E("my/ui/file.ui")):
            pass
            
        class AnotherWidget(E(QtGui.QFrame)):
            pass
    
    """
    ui_path = None
    bases = None
    
    if inspect.isclass(ui_or_widget) and issubclass(ui_or_widget, QtGui.QWidget):
        bases = [ui_or_widget]
    else:
        ui_path = ui_or_widget
        bases = list(uic.loadUiType(ui_path))
    
    name = bases[0].__name__
    bases = tuple([AutoSetup] + bases)
    cls_locals = {"__ui_file__": ui_path}
    return type(name, bases, cls_locals)


def run():
    """Run Qt application"""
    sys.exit(APP.exec_())


#==============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":
    print(__doc__)
